/**************************************************************************
						deployer-yarp.cpp
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/

#include <rtt/os/main.h>
#include <rtt/RTT.hpp>
#include <rtt/Toolkit.hpp>
#include <ocl/TaskBrowser.hpp>
#include "deployer-funcs.hpp"
#include "YarpDeploymentComponent.h"
#include "YarpToolkit.h"
#include <iostream>
#include <string>

using namespace Orocos;
namespace po = boost::program_options;

int ORO_main(int argc, char** argv) {
	RTT::Toolkit::Import(OroYarp::YarpToolkit);

	std::string script;
	std::string name("Deployer");
	std::string path;
	po::variables_map vm;
	int rc = OCL::deployerParseCmdLine(argc, argv, script, name, path, vm);
	
	if (0 != rc) {
		return rc;
	}

	yarp::os::Network yarp;

	OroYarp::YarpDeploymentComponent dc(name);
	dc.properties()->getProperty<std::string>("ComponentPath")->set(path);

	if (!script.empty())
		dc.kickStart(script);
    
	if(!dc.connectYarpPorts())
		log(Warning) << "Some of the YarpPorts were not succesfully connected." << Logger::endl;

	TaskBrowser tb( &dc );
	tb.loop();

	return 0;
}
