/**************************************************************************
					control_component.cpp
					-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
#include "control_component.h"
#include <iostream>
#include <sstream>

namespace OroYarp {

	using namespace RTT;
	using namespace std;

	ControlComponent::ControlComponent(const string& name) :
		TaskContext(name),
		state_port("state"),
		command_port("command"),
		state_delta("delta", "State acceptance threshold.", .1),
		state_max("max", "State max value.", 600),
		control_axis("control_axis", "Control axis index.", 0)
	{
		this->properties()->addProperty(&state_delta);
		this->properties()->addProperty(&state_max);
		this->properties()->addProperty(&control_axis);
		this->ports()->addEventPort(&state_port);
		this->ports()->addPort(&command_port);
		srand(time(NULL));
	}

	bool ControlComponent::startHook() {
		current_goal = ((rand() % int(state_max * 10)) / 10.) - (state_max / 2);
		send(current_goal);
		return true;
	}

	void ControlComponent::updateHook() {
		Logger::In(this->getName());
		// Get Bot state
		state_port.Get(current_state);
		if (current_state.size() < 2) return;
		if (control_axis != 0 && control_axis != 1) {
			log(Warning) << "Control axis must be 0 or 1... setting to 0." << Logger::endl;
			control_axis = 0;
		}
		// Check if goal has been reached
		double x = current_state.at(control_axis);
		double d = abs(current_goal - x);
		if (d < state_delta) {
			log(Info) << "Goal reached with d = " << d << Logger::endl;
			// Generate a new goal
			current_goal = ((rand() % int(state_max * 10)) / 10.) - (state_max / 2);
			send(current_goal);
			return;
		}
		// Else wait
	}

	void ControlComponent::send(double pos) {
			stringstream ss; ss << "set pos " << control_axis << " " << pos;
			command_port.Set(yarp::os::Bottle(ss.str().c_str()));
			log(Info) << "New goal: " << pos << Logger::endl;
	}

}
