/**************************************************************************
					control_component.h
					-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
#ifndef OROYARP_CONTROL_EXAMPLE
#define OROYARP_CONTROL_EXAMPLE

#include <rtt/TaskContexts.hpp>
#include <yarp/os/all.h>

namespace OroYarp {

	/** TaskContext for Yarp FakeBot control. */
	class ControlComponent : public RTT::TaskContext {
	
		public:
			/**
			 * Control Component Constructor.
			 * @param name Component name (default is "Controller").
			 */
			ControlComponent(const std::string& name = "Controller");
		
		protected:
			/**
			 * Generate the first goal and send it.
			 * @return true.
			 */
			virtual bool startHook();
			/** Check if goal has been reached from robot state and generate a new one. */
			virtual void updateHook();
	
		private:
			/**
			 * Send goal position.
			 * @param pos goal
			 */
			void send(double pos);
			/** Control axis index. */
			RTT::Property<int> control_axis;
			/** State acceptance threshold. */
			RTT::Property<double> state_delta;
			/** State max value. */
			RTT::Property<double> state_max;
			/** Input state data port. */
			RTT::ReadDataPort<std::vector<double> > state_port;
			/** Output command data port. */
			RTT::DataPort<yarp::os::Bottle> command_port;
			/** Current goal position. */
			double current_goal;
			/** Current bot position. */
			std::vector<double> current_state;
	};

};

#endif
