/**************************************************************************
						main.cpp
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
#include <rtt/TaskContexts.hpp>
#include <rtt/os/main.h>
#include <rtt/Toolkit.hpp>
#include <ocl/TaskBrowser.hpp>
#include "YarpDeploymentComponent.h"
#include "YarpToolkit.h"
#include "control_component.h"
#include <boost/program_options.hpp>
#include <boost/program_options/positional_options.hpp>
#include <boost/assign/list_of.hpp>
#include <boost/algorithm/string/case_conv.hpp>

namespace po = boost::program_options;

std::map<std::string, RTT::Logger::LogLevel> logMap = boost::assign::map_list_of
	("never",       RTT::Logger::Debug)
	("fatal",       RTT::Logger::Fatal)
	("critical",    RTT::Logger::Critical)
	("error",       RTT::Logger::Error)
	("warning",     RTT::Logger::Warning)
	("info",        RTT::Logger::Info)
	("debug",       RTT::Logger::Debug)
	("realtime",    RTT::Logger::RealTime);

int ORO_main(int argc, char** argv) {
	std::string logLevel("warning");
	std::string viewName("/yarpview/img:i");
	bool autoconnect = false;
	// Options
	po::options_description desc("Allowed options");
	desc.add_options()
		("help,h", "Display help message.")
		("log-level,l", po::value<std::string>(&logLevel), "Level at which to log (case-insensitive).")
		("view,v", po::value<std::string>(&viewName), "Port name of the YarpView.")
		("auto,a", "Automatically create Yarp node and connects Yarp ports.")
	;
	po::variables_map vm;
	po::store(po::parse_command_line(argc, argv, desc), vm);
	po::notify(vm);
	if (vm.count("help")) {
		std::cout << desc << std::endl;
		return 1;
	}
	if (vm.count("auto")) {
		autoconnect = true;
	}

	// Setting Log Level
	boost::algorithm::to_lower(logLevel);
	if (logMap.count(logLevel) <= 0)
		RTT::Logger::Instance()->setLogLevel(RTT::Logger::Warning);
	else
		RTT::Logger::Instance()->setLogLevel(logMap[logLevel]);
	// Importing Yarp Toolkit
	RTT::Toolkit::Import(OroYarp::YarpToolkit);
	// Initialize yarp network
	yarp::os::Network::init();
	assert(yarp::os::Network::checkNetwork());
	// Creating Yarp Deployer
	OroYarp::YarpDeploymentComponent dc;
	// Creating Controller
	OroYarp::ControlComponent cc("Controller");
	dc.addPeer(&cc);
	if (autoconnect) dc.createYarpNode("Controller");
	// Creating Browser
	OCL::TaskBrowser tc(&dc);
	// Connection of Yarp ports
	if (autoconnect) {
		assert(yarp::os::Network::connect("/fakebot/camera", viewName.c_str()));
		assert(yarp::os::Network::connect("/Deployer/Controller/command", "/fakebot/motor/rpc:i"));
		assert(yarp::os::Network::connect("/fakebot/motor/state:o", "/Deployer/Controller/state"));
		// Loop
		cc.start();
	}
	tc.loop();
	return 0;
}
