/**************************************************************************
					Orocos2Yarp.cpp
					-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/

#include "Orocos2Yarp.h"
#include "YarpTypeTemplateConversion.h"

namespace OroYarp {

	using namespace RTT;
	using namespace std; 
 
	YarpNode::YarpNode(const string& name) : TaskContext(name + "_yarp"), node_name(name) {}

	TaskContext* YarpNode::createYarpNode(TaskContext* tc, const string& rname) {
		log(Info) << "Creating YarpNode for component " << tc->getName() << Logger::endl;
		YarpNode* yarp_node = new YarpNode(tc->getName());
		tc->addPeer(yarp_node);
		// Create the corresponding Yarp node
		const DataFlowInterface::Ports& ports = tc->ports()->getPorts();
		for (unsigned int i=0; i < ports.size(); i++){
			log(Info) << "Creating YarpPort for " << tc->getName() + "." + ports[i]->getName() << Logger::endl;
			// Create anticlone for the port
			PortInterface* anticlone(ports[i]->antiClone());
			// Add the anticlone to the ports of the node
			yarp_node->ports()->addPort(anticlone);
			// Connect the port and its anticlone
			ports[i]->connectTo(anticlone);
			// Create the YarpPort
			if (ports[i]->getPortType() == PortInterface::WritePort)
				yarp_node->write_ports.push_back(YarpPortCreator::Instance()->createWritePort("/" + rname + "/" + yarp_node->node_name, anticlone, yarp_node));
			else if (ports[i]->getPortType() == PortInterface::ReadPort)
				yarp_node->read_ports.push_back(YarpPortCreator::Instance()->createReadPort("/" + rname + "/" + yarp_node->node_name, anticlone));
			else if (ports[i]->getPortType() == PortInterface::ReadWritePort)
				yarp_node->rw_ports.push_back(YarpPortCreator::Instance()->createRWPort("/" + rname + "/" + yarp_node->node_name, anticlone, yarp_node));
    	}
    	return yarp_node;
	}

	YarpNode::~YarpNode() {
		for (unsigned int i=0; i < write_ports.size(); i++)
			delete write_ports[i];
		for (unsigned int i=0; i < read_ports.size(); i++)
			delete read_ports[i];
		for (unsigned int i=0; i < rw_ports.size(); i++)
			delete rw_ports[i];
		const DataFlowInterface::Ports& ports = this->ports()->getPorts();
		for (unsigned int i=0; i < ports.size(); i++)
			delete ports[i];
	}

}
