/**************************************************************************
						Orocos2Yarp.h
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/

#ifndef OROCOS_2_YARP_H
#define OROCOS_2_YARP_H

#include <rtt/TaskContexts.hpp>
#include <yarp/os/all.h>
#include "YarpPortCreator.h"

namespace OroYarp {

	/** An Orocos component connected to Yarp, ie an OroYarp component. */
	class YarpNode : public RTT::TaskContext {
	
		public:
			/**
			 * Component constructor.
			 * @param name Orocos component name.
			 */
			YarpNode(const std::string& name);
			/** Component destructor. */
			~YarpNode();
			/**
			 * Creates an OroYarp component binding ports of a TaskContext.
			 * @param tc The Orocos component.
			 * @param rname The name of Orocos root as it appears to Yarp.
			 * @return A pointer to the OroYarp component.
			 */
			static RTT::TaskContext* createYarpNode(RTT::TaskContext* tc, const std::string& rname = "Orocos");
		
		private:
			/** The yarp node name. */
			std::string node_name;
			/** The set of write ports binded to yarp. */
			std::vector<yarp::os::Port*> write_ports;
			/** The set of read ports binded to yarp. */
			std::vector<yarp::os::Port*> read_ports;
			/** The set of RW ports binded to yarp. */
			std::vector<yarp::os::Port*> rw_ports;
	
	};

}
  
#endif
