/**************************************************************************
					YarpToolkit.cpp
					-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
#include "YarpToolkit.h"
#include "YarpTypeConversion.h"
#include "YarpPortCreator.h"
#include "YarpBottleConversion.h"
#ifdef BUILD_KDL
#include "YarpKDLConversion.h"
#endif
#include <rtt/Toolkit.hpp>

OroYarp::YarpPortCreator* OroYarp::YarpPortCreator::instance = 0;

namespace OroYarp {

	std::string YarpToolkitPlugin::getName() { return "Yarp Toolkit"; }

	bool YarpToolkitPlugin::loadTypes(){
    	// Bottles
    	RTT::types()->addType(new BottleTypeInfo());
    	OroYarp::YarpPortCreator::Instance()->registerType<yarp::os::Bottle>("bottle");
		// RTT Types
    	OroYarp::YarpPortCreator::Instance()->registerType<double>("double");
    	OroYarp::YarpPortCreator::Instance()->registerType<int>("int");
    	OroYarp::YarpPortCreator::Instance()->registerType<bool>("bool");
    	OroYarp::YarpPortCreator::Instance()->registerType<std::string>("string");
    	OroYarp::YarpPortCreator::Instance()->registerType<std::vector<double> >("array");
#ifdef BUILD_KDL
    	// KDL
    	OroYarp::YarpPortCreator::Instance()->registerType<KDL::Vector>("vector");
    	OroYarp::YarpPortCreator::Instance()->registerType<KDL::Rotation>("rotation");
    	OroYarp::YarpPortCreator::Instance()->registerType<KDL::Frame>("frame");
    	OroYarp::YarpPortCreator::Instance()->registerType<KDL::Twist>("twist");
       	OroYarp::YarpPortCreator::Instance()->registerType<KDL::Wrench>("wrench");
#endif
    	return true;
	}

	YarpToolkitPlugin YarpToolkit;

}

ORO_TOOLKIT_PLUGIN(OroYarp::YarpToolkit);
