/**************************************************************************
					YarpToolkit.h
					-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/

/**
 * \mainpage Orocos-Yarp Binding Documentation
 *
 * \section intro_sec Introduction
 *
 * The Orocos-Yarp binding package has been released on the RoboTIS project website of 
 * Onera at <a href="http://robotis.onera.fr/orocos">http://robotis.onera.fr/orocos</a>.<br>
 * 
 * It has been designed to automatically export Orocos components data ports to the Yarp framework.
 * An extension has also been made to the DeploymentComponent to accept incoming requests from Yarp nodes.
 * These requests are binded to Orocos State Machine execution, with the possibility to define the FSM parameters.
 *
 * \section install_sec Installation
 *
 * The OroYarp library needs Orocos and Yarp to be installed:
 * <ul>
 * <li>Orocos RTT version 1.10.x,
 * <li>Orocos OCL version 1.10.x,
 * <li>Orocos KDL to build KDL bindings,
 * <li>Boost program options and date time packages (tested with version 1.40),
 * <li>Yarp version 2.2.3 or later.
 * </ul>
 * Installation is based on CMake (2.6 or later). The library will be installed to:
 * <ul>
 * <li>INSTALL_DIR/bin for deployer-yarp binary,
 * <li>INSTALL_DIR/include/robotis for header files,
 * <li>INSTALL_DIR/lib for the binding toolkit.
 * </ul>
 * INSTALL_DIR is by default set to /usr/local.
 *
 * \section copyright Copyright and License
 *
 * Copyright Onera 2009.<br>
 *
 * This file is part of RoboTIS OroYarp package.<br>
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.<br>
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.<br>
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <a href="http://www.gnu.org/licenses/">http://www.gnu.org/licenses/</a>.
 *
 */
 
#ifndef YARP_TOOLKIT_H
#define YARP_TOOLKIT_H

#include <rtt/ToolkitPlugin.hpp>

namespace OroYarp {

	/** The Yarp binding toolkit. */
	class YarpToolkitPlugin : public RTT::ToolkitPlugin {
	
		public:
			/**
			 * The name of the Toolkit.
			 * @return something like "Yarp Toolkit".
			 */
			virtual std::string getName();
			/**
			 * Import the 'bottle' type to Orocos toolkit and register OroYarp bindings.
			 * @return true.
			 */
			virtual bool loadTypes();
			/**
			 * Does nothing.
			 * @return true.
			 */
			virtual bool loadConstructors() { return true; };
			/**
			 * Does nothing.
			 * @return true.
			 */
			virtual bool loadOperators() { return true; };
	};
  
  	/** The Yarp Toolkit instance. */
	extern YarpToolkitPlugin YarpToolkit;

}

#endif

