/**************************************************************************
						YarpTypeTemplateConversion.h
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/

#ifndef YARP_TYPE_TEMPLATE_CONVERSION_H
#define YARP_TYPE_TEMPLATE_CONVERSION_H

#include <yarp/os/all.h>
#include <rtt/TaskContexts.hpp>
#include <boost/function.hpp>

namespace OroYarp {

	using namespace RTT;

	/** Generic Orocos / Yarp type conversion structure. */
	template <typename Type, typename YarpT>
	struct YarpTypeConversion {
		/** The Orocos type. */
		typedef Type OrocosType;
		/** The corresponding Yarp type. */
		typedef YarpT YarpType;
		/**
		 * Converts the Yarp object to Orocos.
		 * @param yarp The Yarp object.
		 * @param orocos The orocos object.
		 * @return false if it does not know how to convert this type of object.
		 */
		static bool copyYarpToOrocos(const YarpType& yarp, OrocosType& orocos) {
			log(Error) << "Failing conversion of YarpType to type " << detail::DataSourceTypeInfo<Type>::getType() << "." << Logger::endl;
			return false;
		};
		/**
		 * Converts the Orocos object to Yarp.
		 * @param yarp The Yarp object.
		 * @param orocos The orocos object.
		 * @return false if it does not know how to convert this type of object.
		 */
		static bool copyOrocosToYarp(const OrocosType& orocos, YarpType& yarp) {
			log(Error) << "Failed conversion of type " << detail::DataSourceTypeInfo<Type>::getType() << " to YarpType." << Logger::endl;
			return false;
		};
	};

	/** Default Orocos / Yarp type conversion structure exports Orocos types to Bottles. */
	template <typename T>
	struct StdYarpTypeConversion: public YarpTypeConversion<T, yarp::os::Bottle> {};

}

#endif
