/**************************************************************************
						test_yarp_scripting.cpp
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009-2010 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
#include <rtt/TaskContexts.hpp>
#include <rtt/scripting/StateMachineTask.hpp>
#include <rtt/SequentialActivity.hpp>
#include <rtt/os/main.h>
#include <rtt/Toolkit.hpp>
#include <yarp/os/all.h>
#include "YarpDeploymentComponent.h"
#include "YarpToolkit.h"
#include <boost/progress.hpp>
#include <boost/thread.hpp>

int ORO_main(int argc, char** argv) {
	RTT::log().setLogLevel(RTT::Logger::Never);
	srand(time(NULL));
	yarp::os::Port p;
	yarp::os::Bottle bottle;
	
	// Importing Yarp Toolkit
	std::cout << "Importing Orocos Yarp Toolkit... " << std::flush;
	RTT::Toolkit::Import(OroYarp::YarpToolkit);
	std::cout << "OK" << std::endl;
	
	// Initialize yarp network
	std::cout << "Initializing Yarp network... " << std::flush;
	yarp::os::Network::init();
	if (!yarp::os::Network::checkNetwork())
		yarp::os::Network::setLocalMode(true);
	assert(p.open("/yarp"));
	std::cout << "OK" << std::endl;
	
	// Creating Yarp Deployer
	std::cout << "Creating Yarp deployer... " << std::flush;
	OroYarp::YarpDeploymentComponent* dc = new OroYarp::YarpDeploymentComponent();
	dynamic_cast<RTT::TaskContext*>(dc)->setActivity(new RTT::SequentialActivity());
	dc->start();
	assert(yarp::os::Network::connect("/yarp", "/Deployer/Scripting"));
	std::cout << "OK" << std::endl;
	
	// Loading the test FSM
	std::cout << "Loading test FSM " 
		<< std::string(STATE_MACHINES_PATH) << "yarpscripting_fsm.osd... " << std::flush;
	assert(dc->scripting()->loadStateMachines(std::string(STATE_MACHINES_PATH) + "yarpscripting_fsm.osd"));
	std::cout << "OK" << std::endl;
	// Checking FSM status
	std::cout << "Checking FSM status... " << dc->scripting()->getStateMachineStatus("test") << std::endl;
	RTT::StateMachineTask* fsm = dynamic_cast<RTT::StateMachineTask*>(dc->getObject("test"));
	
	// Launching test FSM from Yarp
	std::cout << "Activating FSM 'test'..." << std::flush;
	bottle.addString("test.activate()");
	p.write(bottle);
	boost::this_thread::sleep(boost::posix_time::milliseconds(100));
	assert(dc->execution()->isStateMachineActive("test"));
	bottle.clear();
	std::cout << " OK" << std::endl;
	
	int i = rand() % 100 + 1;
	std::cout << "Setting 'test' parameter V_int to " << i << std::flush;
	std::stringstream parami; parami << "test.V_int = " << i;
	bottle.addString(parami.str().c_str());
	p.write(bottle);
	boost::this_thread::sleep(boost::posix_time::milliseconds(100));
	RTT::Attribute<int>* v = fsm->attributes()->getAttribute<int>("V_int");
	assert(v);
	assert(v->get() == i);
	bottle.clear();
	std::cout << " OK" << std::endl;
		
	std::cout << "Starting FSM 'test'..." << std::flush;
	bottle.addString("test.start()");
	p.write(bottle);
	boost::this_thread::sleep(boost::posix_time::milliseconds(100));
	assert(dc->execution()->isStateMachineRunning("test"));
	assert(dc->execution()->inStateMachineState("test", "P"));
	std::cout << " OK" << std::endl;
		
	// Quit
    yarp::os::Network::fini();
    std::cout << "Test Successful." << std::endl;
	return 0;
}
