/**************************************************************************
						yarpbottle_test.cpp
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
#include <rtt/os/main.h>
#include <yarp/os/all.h>
#include "YarpBottleConversion.h"
#include <rtt/marsh/PropertyMarshaller.hpp>
#include <rtt/marsh/PropertyDemarshaller.hpp>
#include <fstream>

#define VOCAB_FIRST VOCAB3('f','i','r')
#define VOCAB_SECOND VOCAB3('s','e','c')

int ORO_main(int argc, char** argv) {
	srand(time(NULL));
	OroYarp::BottleTypeInfo ti;
	std::string filename("bottle_test.cpf");
	
	yarp::os::Bottle b;
	// Add an int to the bottle
	int i = rand() % 100;
	b.addInt(i);
	// Add a double to the bottle
	double d = (rand() % 1000) / 10.;
	b.addDouble(d);
	// Add a string to the bottle
	b.addString("Hello");
	// Add a vocab to the bottle
	int v = (rand() % 2);
	b.addVocab((v == 0) ? VOCAB_FIRST : VOCAB_SECOND);
	// Add a list with a subbottle
	yarp::os::Bottle& bb = b.addList();
	i = rand() % 100;
	bb.addInt(i);
	bb.addString("World");
	
	// Write bottle
	ti.write(std::cout << "Generated bottle ", b) << std::endl;
	// Decompose the bottle to a property bag
	RTT::PropertyMarshaller pout(filename);
	RTT::PropertyBag bag;
	std::cout << "Decomposing the bottle to a PropertyBag in file " << filename << "... " << std::flush;
	assert(ti.decomposeTypeImpl(b, bag));
	pout.serialize(bag);
	pout.flush();
	std::cout << "OK" << std::endl;
	// Display CPF file
	std::ifstream ifs(filename.c_str(), std::ifstream::in);
	while (ifs.good()) std::cout << (char) ifs.get();
	std::cout << std::endl;

	// Read the property bag
	RTT::PropertyDemarshaller pin(filename);
	RTT::PropertyBag inbag;
	std::cout << "Reading file " << filename << " again... " << std::flush;
	assert(pin.deserialize(inbag));
	std::cout << "OK" << std::endl;
	yarp::os::Bottle bq;
	std::cout << "Recomposing the bottle from PropertyBag of file " << filename << "... " << std::flush;
	assert(ti.composeTypeImpl(inbag, bq));
	std::cout << "OK" << std::endl;
	ti.write(std::cout, bq) << std::endl;
	
	// Quit
    std::cout << "Test Successful." << std::endl;
	return 0;
}

