/**************************************************************************
						yarpkdl_test.cpp
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
#include <rtt/TaskContexts.hpp>
#include <rtt/os/main.h>
#include <rtt/Toolkit.hpp>
#include <yarp/os/all.h>
#include "Orocos2Yarp.h"
#include "YarpToolkit.h"
#include <kdl/frames.hpp>
#include <kdl/frames_io.hpp>
#include <kdl/bindings/rtt/toolkit.hpp>

yarp::os::Port p;
yarp::os::Bottle b;

int ORO_main(int argc, char** argv) {
	RTT::log().setLogLevel(RTT::Logger::Never);
	srand(time(NULL));
	
	// Importing Yarp Toolkit
	std::cout << "Importing Orocos Yarp Toolkit... " << std::flush;
	RTT::Toolkit::Import(KDL::KDLToolkit);
	RTT::Toolkit::Import(OroYarp::YarpToolkit);
	std::cout << "OK" << std::endl;
	
	// Initialize yarp network
	std::cout << "Initializing Yarp network... " << std::flush;
	yarp::os::Network::init();
	if (!yarp::os::Network::checkNetwork())
		yarp::os::Network::setLocalMode(true);
	std::cout << "OK" << std::endl;
	assert(p.open("/yarp"));

	// Create TestComponent
    std::cout << "Creating Test component... " << std::flush;
    RTT::TaskContext* test = new RTT::TaskContext("Test");
	RTT::ReadDataPort<KDL::Vector> in_vector("in_vector"); test->ports()->addPort(&in_vector);
	RTT::WriteDataPort<KDL::Vector> out_vector("out_vector"); test->ports()->addPort(&out_vector);
	RTT::ReadDataPort<KDL::Rotation> in_rotation("in_rotation"); test->ports()->addPort(&in_rotation);
	RTT::WriteDataPort<KDL::Rotation> out_rotation("out_rotation"); test->ports()->addPort(&out_rotation);
	RTT::ReadDataPort<KDL::Frame> in_frame("in_frame"); test->ports()->addPort(&in_frame);
	RTT::WriteDataPort<KDL::Frame> out_frame("out_frame"); test->ports()->addPort(&out_frame);
	RTT::ReadDataPort<KDL::Twist> in_twist("in_twist"); test->ports()->addPort(&in_twist);
	RTT::WriteDataPort<KDL::Twist> out_twist("out_twist"); test->ports()->addPort(&out_twist);
	RTT::ReadDataPort<KDL::Wrench> in_wrench("in_wrench"); test->ports()->addPort(&in_wrench);
	RTT::WriteDataPort<KDL::Wrench> out_wrench("out_wrench"); test->ports()->addPort(&out_wrench);
	std::cout << "OK" << std::endl;

	// Launching TestComponent
	std::cout << "Starting Test component... " << std::flush;
	assert(test->start());
	std::cout << "OK" << std::endl;
	
	// Binding to Yarp
	std::cout << "Binding Test component to Yarp... " << std::flush;
	RTT::TaskContext* yarpnode = OroYarp::YarpNode::createYarpNode(test);
	assert(yarpnode);
	assert(connectPorts(yarpnode, test));
	assert(yarpnode->start());
	std::cout << "OK" << std::endl;
	
	// KDL::VECTOR
	std::cout << "TESTING CONVERSION OF KDL::VECTOR" << std::endl;
	// Yarp --> Orocos
	KDL::Vector v;
	for (int i = 0; i < 3; i++) v[i] = (rand() % 1000) / 10.;
	assert(yarp::os::Network::connect("/yarp", "/Orocos/Test/in_vector"));
	std::cout << "Sending '" << v << "' from Yarp to Orocos... " << std::flush;
	for (int i = 0; i < 3; i++) b.addDouble(v[i]);
	p.write(b);
	assert(in_vector.Get() == v);
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/yarp", "/Orocos/Test/in_vector");
	// Orocos --> Yarp
	for (int i = 0; i < 3; i++) v[i] = (rand() % 1000) / 10.;
	assert(yarp::os::Network::connect("/Orocos/Test/out_vector", "/yarp"));
	std::cout << "Sending '" << v << "' from Orocos to Yarp... " << std::flush;
	out_vector.Set(v);
	p.read(b);
	for (int i = 0; i < 3; i++)	assert(v[i] == b.get(i).asDouble());
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/Orocos/Test/out_vector", "/yarp");
	
	// KDL::ROTATION
	std::cout << "TESTING CONVERSION OF KDL::ROTATION" << std::endl;
	// Yarp --> Orocos
	KDL::Rotation r;
	for (int i = 0; i < 3; i++) for (int j = 0; j < 3; j++) r(i,j) = (rand() % 1000) / 10.;
	assert(yarp::os::Network::connect("/yarp", "/Orocos/Test/in_rotation"));
	std::cout << "Sending '" << r << "' from Yarp to Orocos... " << std::flush;
	for (int i = 0; i < 3; i++) for (int j = 0; j < 3; j++) b.addDouble(r(i,j));
	p.write(b);
	assert(in_rotation.Get() == r);
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/yarp", "/Orocos/Test/in_rotation");
	// Orocos --> Yarp
	for (int i = 0; i < 3; i++) for (int j = 0; j < 3; j++) r(i,j) = (rand() % 1000) / 10.;
	assert(yarp::os::Network::connect("/Orocos/Test/out_rotation", "/yarp"));
	std::cout << "Sending '" << r << "' from Orocos to Yarp... " << std::flush;
	out_rotation.Set(r);
	p.read(b);
	for (int i = 0; i < 3; i++) for (int j = 0; j < 3; j++)	assert(r(i,j) == b.get(3*i+j).asDouble());
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/Orocos/Test/out_rotation", "/yarp");
	
	// KDL::FRAME
	std::cout << "TESTING CONVERSION OF KDL::FRAME" << std::endl;
	// Yarp --> Orocos
	KDL::Frame f;
	for (int i = 0; i < 3; i++) f.p[i] = (rand() % 1000) / 10.;
	for (int i = 0; i < 3; i++) for (int j = 0; j < 3; j++) f.M(i,j) = (rand() % 1000) / 10.;
	assert(yarp::os::Network::connect("/yarp", "/Orocos/Test/in_frame"));
	std::cout << "Sending '" << f << "' from Yarp to Orocos... " << std::flush;
	for (int i = 0; i < 3; i++) for (int j = 0; j < 3; j++) b.addDouble(f.M(i,j));
	for (int i = 0; i < 3; i++) b.addDouble(f.p[i]);
	p.write(b);
	assert(in_frame.Get() == f);
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/yarp", "/Orocos/Test/in_frame");
	// Orocos --> Yarp
	for (int i = 0; i < 3; i++) for (int j = 0; j < 3; j++) f.M(i,j) = (rand() % 1000) / 10.;
	for (int i = 0; i < 3; i++) f.p[i] = (rand() % 1000) / 10.;
	assert(yarp::os::Network::connect("/Orocos/Test/out_frame", "/yarp"));
	std::cout << "Sending '" << f << "' from Orocos to Yarp... " << std::flush;
	out_frame.Set(f);
	p.read(b);
	for (int i = 0; i < 3; i++) for (int j = 0; j < 3; j++)	assert(f.M(i,j) == b.get(3*i+j).asDouble());
	for (int i = 0; i < 3; i++) assert(f.p[i] == b.get(9+i).asDouble());
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/Orocos/Test/out_frame", "/yarp");
	
	// KDL::TWIST
	std::cout << "TESTING CONVERSION OF KDL::TWIST" << std::endl;
	// Yarp --> Orocos
	KDL::Twist t;
	for (int i = 0; i < 6; i++) t[i] = (rand() % 1000) / 10.;
	assert(yarp::os::Network::connect("/yarp", "/Orocos/Test/in_twist"));
	std::cout << "Sending '" << t << "' from Yarp to Orocos... " << std::flush;
	for (int i = 0; i < 6; i++) b.addDouble(t[i]);
	p.write(b);
	assert(in_twist.Get() == t);
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/yarp", "/Orocos/Test/in_twist");
	// Orocos --> Yarp
	for (int i = 0; i < 6; i++) t[i] = (rand() % 1000) / 10.;
	assert(yarp::os::Network::connect("/Orocos/Test/out_twist", "/yarp"));
	std::cout << "Sending '" << t << "' from Orocos to Yarp... " << std::flush;
	out_twist.Set(t);
	p.read(b);
	for (int i = 0; i < 6; i++)	assert(t[i] == b.get(i).asDouble());
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/Orocos/Test/out_twist", "/yarp");
	
	// KDL::WRENCH
	std::cout << "TESTING CONVERSION OF KDL::WRENCH" << std::endl;
	// Yarp --> Orocos
	KDL::Wrench w;
	for (int i = 0; i < 6; i++) w[i] = (rand() % 1000) / 10.;
	assert(yarp::os::Network::connect("/yarp", "/Orocos/Test/in_wrench"));
	std::cout << "Sending '" << w << "' from Yarp to Orocos... " << std::flush;
	for (int i = 0; i < 6; i++) b.addDouble(w[i]);
	p.write(b);
	assert(in_wrench.Get() == w);
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/yarp", "/Orocos/Test/in_wrench");
	// Orocos --> Yarp
	for (int i = 0; i < 6; i++) w[i] = (rand() % 1000) / 10.;
	assert(yarp::os::Network::connect("/Orocos/Test/out_wrench", "/yarp"));
	std::cout << "Sending '" << w << "' from Orocos to Yarp... " << std::flush;
	out_wrench.Set(w);
	p.read(b);
	for (int i = 0; i < 6; i++)	assert(w[i] == b.get(i).asDouble());
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/Orocos/Test/out_wrench", "/yarp");
	
	// Quit
	delete yarpnode;
	delete test;
    yarp::os::Network::fini();
    std::cout << "Test Successful." << std::endl;
	return 0;
}

