/**************************************************************************
						yarprw_test.cpp
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2010 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
#include <rtt/TaskContexts.hpp>
#include <rtt/os/main.h>
#include <rtt/Toolkit.hpp>
#include <yarp/os/all.h>
#include "Orocos2Yarp.h"
#include "YarpToolkit.h"

yarp::os::Port p;
yarp::os::Bottle b;

int ORO_main(int argc, char** argv) {
	RTT::log().setLogLevel(RTT::Logger::Never);
	yarp::os::Network yarpN;
	srand(time(NULL));
	
	// Importing Yarp Toolkit
	std::cout << "Importing Orocos Yarp Toolkit... " << std::flush;
	RTT::Toolkit::Import(OroYarp::YarpToolkit);
	std::cout << "OK" << std::endl;
	
	// Initialize yarp network
	std::cout << "Initializing Yarp network... " << std::flush;
	yarp::os::Network::init();
	if (!yarp::os::Network::checkNetwork())
		yarp::os::Network::setLocalMode(true);
	assert(p.open("/yarp"));
	std::cout << "OK" << std::endl;

	// Create TestComponent
    std::cout << "Creating Test component... " << std::flush;
    RTT::TaskContext* test = new RTT::TaskContext("Test");
	RTT::DataPort<int> io_int("io_int"); test->ports()->addPort(&io_int);
	std::cout << "OK" << std::endl;

	// Launching TestComponent
	std::cout << "Starting Test component... " << std::flush;
	assert(test->start());
	std::cout << "OK" << std::endl;
	
	// Binding to Yarp
	std::cout << "Binding Test component to Yarp... " << std::flush;
	RTT::TaskContext* yarpnode = OroYarp::YarpNode::createYarpNode(test);
	assert(yarpnode);
	assert(connectPorts(yarpnode, test));
	assert(yarpnode->start());
	std::cout << "OK" << std::endl;
	
	std::cout << "TESTING CONVERSION OF INT" << std::endl;
	// Yarp --> Orocos
	int i = rand() % 100;
	assert(yarp::os::Network::connect("/yarp", "/Orocos/Test/io_int"));
	std::cout << "Sending '" << i << "' from Yarp to Orocos... " << std::flush;
	b.add(i);
	p.write(b);
	assert(io_int.Get() == i);
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/yarp", "/Orocos/Test/io_int");
	// Orocos --> Yarp
	i = rand() % 100;
	assert(yarp::os::Network::connect("/Orocos/Test/io_int", "/yarp"));
	std::cout << "Sending '" << i << "' from Orocos to Yarp... " << std::flush;
	io_int.Set(i);
	p.read(b);
	assert(b.get(0) == yarp::os::Value(i));
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/Orocos/Test/io_int", "/yarp");
			
	// Quit
	delete yarpnode;
	delete test;
    yarp::os::Network::fini();
    std::cout << "Test Successful." << std::endl;
	return 0;
}

